@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2021 Ioannis Kosmidis
+# Copyright (C) 2017- Ioannis Kosmidis
 
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,16 @@ brglm_control <- brglmControl
 #' @export
 brglm_fit <- brglmFit
 
+#' @rdname mdyplFit
+#' @export
+mdypl_fit <- mdyplFit
+
+#' @rdname mdyplControl
+#' @export
+mdypl_control <- mdyplControl
+
+
+
 #### Method conventions
 
 #' @rdname residuals.brmultinom
@@ -30,3 +40,4 @@ brglm_fit <- brglmFit
 #' @export
 residuals.bracl <- residuals.brmultinom
 
+
